<?php
namespace Service\Mongo\Types;

use DateTime;
use DateTimeZone;
use Exception;
use InvalidArgumentException;
use MongoDB\BSON\UTCDatetime;

class DateType extends Type {

    /**
     * Converts a value to a DateTime.
     * Supports microseconds
     *
     * @throws InvalidArgumentException if $value is invalid
     *
     * @param  mixed $value \DateTime|\MongoDB\BSON\UTCDatetime|int|float
     *
     * @return DateTime
     */
    public static function getDateTime($value): DateTime
    {
        $datetime = false;
        $exception = null;
        if ($value instanceof DateTime) {
            return $value;
        } elseif ($value instanceof UTCDatetime) {
            $datetime = $value->toDateTime();
        } elseif (is_numeric($value)) {
            $seconds = $value;
            $microseconds = 0;
            if (false !== strpos($value, '.')) {
                list($seconds, $microseconds) = explode('.', $value);
                $microseconds = (int)str_pad((int)$microseconds, 6, '0'); // ensure microseconds
            }
            $datetime = self::craftDateTime($seconds, $microseconds);
        } elseif (is_string($value)) {
            try {
                $datetime = new DateTime($value, self::getUTC());
            } catch (Exception $e) {
                dd($e->getMessage());
                $exception = $e;
            }
        }
        if ($datetime === false) {
            throw new \InvalidArgumentException(sprintf('Could not convert %s to a date value', is_scalar($value) ? '"' . $value . '"' : gettype($value)), 0, $exception);
        }
        return $datetime;
    }

    /**
     * @return DateTimeZone
     */
    public static function getUTC(): DateTimeZone
    {
        return new DateTimeZone("UTC");
    }

    /**
     * @throws Exception
     */
    private static function craftDateTime($seconds, $microseconds = 0) {
        $datetime = new DateTime('now', new DateTimeZone("UTC"));
        $datetime->setTimestamp($seconds);
        if ($microseconds > 0) {
            $datetime = DateTime::createFromFormat('Y-m-d H:i:s.u', $datetime->format('Y-m-d H:i:s') . '.' . $microseconds, self::getUTC());
        }
        return $datetime;
    }

    /**
     * @param $value
     * @return UTCDatetime|null
     */
    public function convertToDatabaseValue($value): ?UTCDatetime
    {
        if ($value === null || $value instanceof UTCDatetime) {
            return $value;
        }
        $datetime = self::getDateTime($value);
        return new UTCDatetime($datetime->format('U').'000');
    }

    /**
     * @param $value
     * @return DateTime|null
     */
    public function convertToPHPValue($value): ?DateTime
    {
        if ($value === null) {
            return null;
        }
        return self::getDateTime($value);
    }


}
